/*
 * Diplomka.c
 *
 * Created: 19.11.2011 15:42:35
 *  Author: Bus Ondrej
 */ 

#include <avr/io.h>																	//zakladni knihovna mikrokotroleru
#include <avr/interrupt.h>															//preruseni
#include <util/delay.h>																//pro _delay_ms()
#include <avr/eeprom.h>																//knihovna pro praci s EEPROM
#include <avr/pgmspace.h>															//vyzadovano usbdrv.h
#include <avr/sleep.h>																//usporny rezim
#include "usbdrv.h"																	//knihovna USB
#include "dds_lbr.h"																//knihovna DDS
#include "uc_lbr.h"																	//knihovna mikrokontroleru

#define F_CPU			12000000UL													//pracovni kmitocet procesoru
#define TRUE			1															//pomocne konstanty
#define FALSE			0
#define HARDWARE		1
#define SOFTWARE		0
#define PRESET_1		0
#define PRESET_2		1
#define PRESET_3		2
#define PRESET_4		3

																					//EEPROM konstanty
unsigned long long int ee_frequency_presets[8] EEMEM = {F_10_MHZ, 2*F_10_MHZ, 3*F_10_MHZ, 4*F_10_MHZ, MAX_OUT_POWER, MAX_OUT_POWER, MAX_OUT_POWER, MAX_OUT_POWER};
unsigned int ee_out_power_corr_const[13] EEMEM = {8000, 8000, 6800, 6800, 7100, 7500, 7900, 8100, 8500, 9000, 9200, 9400, 10000};
unsigned char ee_fsample = 0;
unsigned char ee_pll_mult_control EEMEM = MULTIPLIER_PLL;
unsigned char ee_charge_pump_curr EEMEM = CH_PUMP_150U;
unsigned char ee_power_corr_control EEMEM = ENABLE;

unsigned long long int frequency_presets[8];										//obsahuje 4 prednastavene kmitocty a k nim prislusne 4 vykony, ktere lze volit pomoci prepinacu na panelu
unsigned int out_power_corr_const[13];												//obsahuje korekcni konstanty vystupniho vykonu vyctene z EEPROM
unsigned char fsample;																//obsahuje informace o zvolenem taktovacim kmitoctu
unsigned char pll_mult_control;														//0 = PLL vypnuta, 4-20 nasobek
unsigned char charge_pump_curr;														//proud nabojove pumpy
unsigned char power_corr_control;													//aktivace ci deaktivace korekce vykonu

unsigned char dds_init_request = 0;													//pomocna promenna pro reinicializaci DDS po vypadku napajeni

unsigned int out_power = MAX_OUT_POWER;												//vystupni vykon v uW
unsigned int out_power_corr = MAX_OUT_POWER;										//korigovany vystupni vykon
unsigned long long int frequency = F_10_MHZ;										//frekvence v mHz
unsigned char clk_source = INTERNAL;												//informace o zdroji hodinoveho kmitoctu (interni/externi)
unsigned char control = SOFTWARE;													//urcuje, zda-li bude DDS ovladana pomoci USB (SOFTWARE) nebo predniho panelu (HARDWARE)

unsigned char preset_num = PRESET_1;												//cislo predvolby A a B
unsigned char preset_sel = ROT_CTRL_A;												//vybrana predvolba (A nebo B)

unsigned char tx_data[8];															//buffer pro odesilani dat pres USB
unsigned char rx_data[8];															//buffer pro prijem dat pres USB

static unsigned char currentAddress;												//pomocna promenna pro komunikaci USB
static unsigned char bytesRemaining;												//pomocna promenna pro komunikaci USB
unsigned char new_usb_data_flag = 0;												//indikuje, ze prisla nova data z USB
unsigned char ack = 0;																//potvrzovaci bit spravneho prijmu dat z PC, po prijeti a zpracovani dat dojde k inverzi bitu

PROGMEM char usbHidReportDescriptor[38] = {		// USB report descriptor
	0x06, 0x00, 0xff,			// USAGE_PAGE (Generic Desktop) 
    0x09, 0x01,					// USAGE (Vendor Usage 1) 
    0xa1, 0x01,					// COLLECTION (Application) 
    0x19, 0x01,					//   USAGE_MINIMUM (0), minimalni pocet bajtu, ktere lze prenest
    0x29, 0x08,					//   USAGE_MAXIMUM (8), maximalni pocet bajtu, ktere lze prenest
    0x15, 0x00,					//   LOGICAL_MINIMUM (0), min. hodnota bajtu
    0x26, 0xff, 0x00,			//   LOGICAL_MAXIMUM (255), max hodnota bajtu 
    0x75, 0x08,					//   REPORT_SIZE (8) 
    0x95, 0x08,					//   REPORT_COUNT (8), pocet bajtu, ktere se odeslou naraz
    0x81, 0x02,					//   INPUT (Data,Var,Abs) 
	
    0x19, 0x01,					//   USAGE_MINIMUM (0) 
    0x29, 0x08,					//   USAGE_MAXIMUM (8) 
    0x15, 0x00,					//   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,			//   LOGICAL_MAXIMUM (255)
    0x75, 0x08,					//   REPORT_SIZE (8) 
    0x95, 0x08,					//   REPORT_COUNT (8)
    0x91, 0x02,					//   OUTPUT (Data,Var,Abs) 
    0xc0						// END_COLLECTION
};

unsigned long long int ee_frequency_presets_read(unsigned char number)				//umoznuje cteni prednastavenych kmitoctu z EEPROM (0 - 3 kmitocty, 4 - 7 vykony)
{
	unsigned long long int preset;	
	eeprom_read_block((void*)&preset, (const void*)&ee_frequency_presets[number], sizeof(unsigned long long int));
	return preset;
}

unsigned int ee_power_corr_read(unsigned char number)								//umoznuje cteni korekcnich konstant vystupniho vykonu z EEPROM
{
	unsigned int corr;	
	eeprom_read_block((void*)&corr, (const void*)&ee_out_power_corr_const[number], sizeof(unsigned int));
	return corr;
}

void ee_frequency_presets_write(unsigned long long value, unsigned char number)		//zapis prednastavenych kmitoctu do EEPROM (0 - 3 kmitocty, 4 - 7 vykony)
{
	eeprom_write_block((void*)&value, (const void*)&ee_frequency_presets[number], sizeof(unsigned long long int));
}

void ee_power_corr_write(unsigned int value, unsigned char number)					//zapis korekcnich konstant do EEPROM
{
	eeprom_write_block((void*)&value, (const void*)&ee_out_power_corr_const[number], sizeof(unsigned int));
}

void load_eeprom(void)																//kopiruje data z EEPROM do SRAM pro rychlejsi pristup
{
	unsigned char i;
	
	for(i = 0; i < 8; i++) frequency_presets[i] = ee_frequency_presets_read(i);		//nacteni predvoleb kmitoctu, zkontrolovat !!!!!!!!!!!
	for(i = 0; i < 13; i++) out_power_corr_const[i] = ee_power_corr_read(i);		//nacteni korekcnich konstant vystupniho vykonu
	
	fsample = eeprom_read_byte((const void*)&ee_fsample);	
	pll_mult_control = eeprom_read_byte((const void*)&ee_pll_mult_control);			//nacteni nastaveni PLL (ENABLE = PLL nasobic 5x, DISABLE = vstup 400 MHz)
	charge_pump_curr = eeprom_read_byte((const void*)&ee_charge_pump_curr);			//nacteni hodnoty proudu nabojove pumpy z EEPROM
	power_corr_control = eeprom_read_byte((const void*)&ee_power_corr_control);		//nacteni nastaveni korekce vykonu z EEPROM
}

void out_power_corr_func(void)														//koriguje vystupni vykon podle konstant z EEPROM, hodnoty pro mezilehle kmitocty jsou interpolovany
{	
	unsigned char freq_meg = (frequency / 1000000000);
	unsigned long int temp_1;
	unsigned char temp_2;
		
	if(freq_meg >= 120) temp_1 = out_power_corr_const[12];							//pro kmitocty nad 120 MHz pouzij korekcni konstantu pro 120 MHz
	else if(out_power_corr_const[freq_meg / 10] > out_power_corr_const[freq_meg / 10 + 1])		//aproximuje korekcni konstantu pro mezilehle kmitocty
	{
		temp_1 = out_power_corr_const[freq_meg / 10] - out_power_corr_const[freq_meg / 10 + 1];
		temp_1 += 5;
		temp_1 /= 10;
		temp_2 = freq_meg / 10 + 1;
		temp_2 *= 10;
		temp_1 *= temp_2 - freq_meg;
		temp_1 += out_power_corr_const[freq_meg / 10 + 1];
	}
	else
	{
		temp_1 = out_power_corr_const[freq_meg / 10 + 1] - out_power_corr_const[freq_meg / 10];
		temp_1 += 5;
		temp_1 /= 10;
		temp_2 = freq_meg / 10;
		temp_2 *= 10;
		temp_1 *= freq_meg - temp_2;
		temp_1 += out_power_corr_const[freq_meg / 10];
	}
	
	temp_1 *= out_power;
	temp_1 /= MAX_OUT_POWER;
	out_power_corr = temp_1;
}

unsigned char usbFunctionWrite(unsigned char *data, unsigned char len)				//funkce pro prijem dat pres USB
{
 	unsigned char i;
    if(len > bytesRemaining)                
    	len = bytesRemaining;														//pokud je poslano vic bajtu, nez muzeme prijmout
    bytesRemaining -= len;
    for(i = 0; i < len; i++)
        rx_data[currentAddress++] = data[i];
		
	new_usb_data_flag = 1;															//prijata nova data

    return bytesRemaining == 0;														//vrati 1, pokud jsou prijata vsechna data
}

usbMsgLen_t usbFunctionSetup(unsigned char data[8])									//ridici funkce USB
{
    usbRequest_t *rq = (void *)data;		   										//cast to structured data for parsing
    switch(rq->bRequest){
		case USBRQ_HID_SET_REPORT:
    	    currentAddress = 0;			        									//inicializace indexu
    	    bytesRemaining = rq->wLength.word;  									//nastavit velikost dat
    	    if(bytesRemaining > sizeof(rx_data)) 									//limit na velikost bufferu
     	       bytesRemaining = sizeof(rx_data);
       		return USB_NO_MSG;        												//pouzij funkci usbFunctionWrite()
    }
    return 0;                               										//ignorovat neznamy pozadavek
}

void dds_set(void)																	//nastavi DDS podle pomocnych promennych (frequency, out_power), nastavi prepinac podle zdroje kmitoctu
{
	if(clk_source == EXTERNAL)
	{
		if(GET_BIT(CLK_CONTROL_PORT, CLK_CONTROL_PIN) == INTERNAL)
		{
			clk_control(EXTERNAL);													//nastavi externi zdroj hodinoveho signalu, pouze pokud doslo ke zmene
			led_extclk(LED_ON);														//rozsviceni LED na panelu
		}		
	}	
	else
	{
		if(GET_BIT(CLK_CONTROL_PORT, CLK_CONTROL_PIN) == EXTERNAL)
		{
			clk_control(INTERNAL);													//nastavi interni zdroj hodinoveho signalu					
			led_extclk(LED_OFF);													//zhasnuti LED na panelu
		}

		if(fsample) pll_mult_control = 4;											//podle taktovaci frekvence nastavi PLL na 4x
		else pll_mult_control = MULTIPLIER_PLL;										//nebo 5x
	}
	
	dds_multiplier(pll_mult_control, charge_pump_curr);								//nastavi PLL nasobic

	if(dds_frequency(frequency, fsample) == 0)										//nastavi kmitocet, pokud je mimo rozsah nastavi 10MHz
	{
		frequency = F_10_MHZ;
		dds_frequency(frequency, fsample);
	}
	
	if(power_corr_control == ENABLE)
	{
		out_power_corr_func();														//korekce vykonu (pouze pokud je povolena)		
	}
	else
	{
		out_power_corr = out_power;
	}
	
	if(dds_power_uw(out_power_corr) == 0)											//nastavi vykon, pokud je mimo rozsah nastavi maximalni
	{
		out_power = MAX_OUT_POWER;
		if(power_corr_control == ENABLE) out_power_corr_func();
		else out_power_corr = out_power;
		dds_power_uw(out_power_corr);
	}
}

//-----------------------------------------------------------------------------------------------------------------------
// paket prijaty z PC ma tvar
// |   7   |   6   |  5  |  4  |  3  |  2  |  1  |         0           |
// |powerH |powerL |freqH|freq |freq |freq |freqL|00|#R/W|cislo prikazu|
//
// jednotlive prikazy:
// 0      : okamzite nastaveni kmitoctu a vykonu
// 1 - 4  : ulozeni predvoleb kmitoctu a vykonu
// 5 - 17 : nastaveni korekcnich konstant pro kmitocty 0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120 MHz, hodnota frekvence je ignorovana

// prikaz 18:
// |  7 |  6 |  5 |  4 |  3	nastaveni korekce vykonu     |                        2 nastaveni PLL                       |    1 zdroj kmitoctu     |    0 prikaz a nastaveni rizeni     |
// |0x00|0x00|0x00|0x00|000000|power_corr_control|#R/W(3)|0|charge_pump_currH|charge_pump_currL|pll_mult_control|#R/W(2)|000000|clk_cource|#R/W(1)|0|control|#R/W(0)|cislo prikazu (18)|
//-----------------------------------------------------------------------------------------------------------------------

void usb_read(void)																	//funkce pro zpracovani dat prijatych z PC
{
	unsigned char temp = 0;
	
	if((rx_data[0] & 0x1F) == 0)													//cislo prikazu 0
	{						
		if(GET_BIT(rx_data[0], 5) == 1)
		{	
			if(control == SOFTWARE)
			{
				frequency  = rx_data[5];											//zapsani hodnoty kmitoctu do pomocne promenne
				frequency <<= 8;
				frequency |= rx_data[4];
				frequency <<= 8;
				frequency |= rx_data[3];
				frequency <<= 8;
				frequency |= rx_data[2];
				frequency <<= 8;
				frequency |= rx_data[1];
		
				out_power  = rx_data[7];											//zapsani hodnoty vykonu do pomocne promenne
				out_power <<= 8;
				out_power |= rx_data[6];
			}			
				
			dds_set();																//nastaveni DDS
				
			ack ^= 1;																//zmena ack bitu (inverze)		
		}
				
		usb_write(0);																//odeslani dat do PC
	}
	else if(((rx_data[0] & 0x1F) >= 1) && ((rx_data[0] & 0x1F) <= 4))				//rezim zapisu predvoleb kmitoctu (cislo prikazu 1 - 4)
	{				
		if(GET_BIT(rx_data[0], 5) == 1)
		{
			rx_data[0] = rx_data[0] & 0x1F;
				
			frequency_presets[rx_data[0] - 1]  = rx_data[5];						//zapis predvolby kmitoctu
			frequency_presets[rx_data[0] - 1] <<= 8;
			frequency_presets[rx_data[0] - 1] |= rx_data[4];
			frequency_presets[rx_data[0] - 1] <<= 8;
			frequency_presets[rx_data[0] - 1] |= rx_data[3];
			frequency_presets[rx_data[0] - 1] <<= 8;
			frequency_presets[rx_data[0] - 1] |= rx_data[2];
			frequency_presets[rx_data[0] - 1] <<= 8;
			frequency_presets[rx_data[0] - 1] |= rx_data[1];
			
			frequency_presets[rx_data[0] + 3]  = rx_data[7];						//zapis hodnoty vykonu
			frequency_presets[rx_data[0] + 3] <<= 8;
			frequency_presets[rx_data[0] + 3] |= rx_data[6];
																					//ulozeni do EEPROM
			if(frequency_presets[rx_data[0] - 1] != ee_frequency_presets_read(rx_data[0] - 1)) 
				ee_frequency_presets_write(frequency_presets[rx_data[0] - 1], (rx_data[0] - 1));
			if(frequency_presets[rx_data[0] + 3] != ee_frequency_presets_read(rx_data[0] + 3)) 
				ee_frequency_presets_write(frequency_presets[rx_data[0] + 3], (rx_data[0] + 3));			
				
			dds_set();																//nastaveni DDS
					
			ack ^= 1;																//zmena ack bitu (inverze)		
		}

		usb_write(rx_data[0] & 0x1F);												//odeslani dat do PC
	}
	else if(((rx_data[0] & 0x1F) >= 5) && ((rx_data[0] & 0x1F) <= 17))				//rezim zapisu korekcnich konstant (cislo prikazu 5 - 17)
	{
		if(GET_BIT(rx_data[0], 5) == 1)
		{
			rx_data[0] = rx_data[0] & 0x1F;
				
			out_power_corr_const[rx_data[0] - 5]  = rx_data[7];						//zapis prislusne korekcni konstanty
			out_power_corr_const[rx_data[0] - 5] <<= 8;
			out_power_corr_const[rx_data[0] - 5] |= rx_data[6];
																					//ulozeni do EEPROM
			if(out_power_corr_const[rx_data[0] - 5] != ee_power_corr_read(rx_data[0] - 5)) 
				ee_power_corr_write(out_power_corr_const[rx_data[0] - 5], (rx_data[0] - 5));
				
				
			dds_set();																//nastaveni DDS

			ack ^= 1;																//zmena ack bitu (inverze)			
		}

		usb_write(rx_data[0] & 0x1F);												//odeslani dat do PC
	}
	else if((rx_data[0] & 0x1F) == 18)
	{
		if(GET_BIT(rx_data[0], 5) == 1)												//nastaveni zpusobu ovladani (HARDWARE/SOFTWARE)
		{
			if(GET_BIT(rx_data[0], 6) == HARDWARE) control = HARDWARE;				//zapsani informace o zpusobu ovladani		
			else control = SOFTWARE;
			
			temp = 1;
		}
		
		if(GET_BIT(rx_data[1], 0) == 1)												//nastaveni informace o zdroji kmitoctu (INTERNAL/EXTERNAL)
		{
			if(GET_BIT(rx_data[1], 1) == EXTERNAL) clk_source = EXTERNAL;			//zapsani informace o zdroji kmitoctu
			else clk_source = INTERNAL;
			
			temp = 1;
		}
			
		if(GET_BIT(rx_data[2], 0) == 1)												//nastaveni PLL
		{			
			pll_mult_control = (rx_data[2] >> 1) & 0x0F;							//deaktivace ci nastaveni PLL
			
																					//nastaveni nabojove pumpy (00=75uA, 01=100uA, 10=125uA, 11=150uA)
			charge_pump_curr = (GET_BIT(rx_data[2], 6) << 1) | GET_BIT(rx_data[2], 5);
																					//zapis nastaveni PLL do EEPROM (pouze pokud se zapisuje jina hodnota)
			if(pll_mult_control != (eeprom_read_byte((const void*)&ee_pll_mult_control)))
				eeprom_write_byte((const void*)&ee_pll_mult_control, pll_mult_control);
			if(charge_pump_curr != (eeprom_read_byte((const void*)&ee_charge_pump_curr)))
				eeprom_write_byte((const void*)&ee_charge_pump_curr, charge_pump_curr);
				
			temp = 1;
		}
		
		if(GET_BIT(rx_data[3], 0) == 1)												//aktivace/deaktivace korekce vykonu
		{
			if(GET_BIT(rx_data[3], 1) == ENABLE) power_corr_control = ENABLE;
			else power_corr_control = DISABLE;
																					//ulozeni do EEPROM
			if(power_corr_control != (eeprom_read_byte((const void*)&ee_power_corr_control)))
			eeprom_write_byte((const void*)&ee_power_corr_control, power_corr_control);
			
			temp = 1;
		}
		
		if(GET_BIT(rx_data[4], 0) == 1)												//nastaveni fsample, fsample = 0 (400 MHz) nebo fsample = 1 (320 MHz)
		{
			fsample = GET_BIT(rx_data[4], 1);

			if(fsample != (eeprom_read_byte((const void*)&ee_fsample)))				//ulozeni do EEPROM
			eeprom_write_byte((const void*)&ee_fsample, fsample);
			
			temp = 1;
		}
		
		if(temp)																	//pouze pokud doslo k zapisu
		{
			dds_set();																//nastaveni DDS
							
			ack ^= 1;																//zmena ack bitu (inverze)
		}
		
		usb_write(0);																//odeslani dat do PC
	}
}

//-----------------------------------------------------------------------------------------------------------------------
// paket odeslany do PC ma tvar
// |				         	      	        7                                                     |    6    |   5   |  4  |  3  |  2  |  1  |  0  |
// |power_corr_control|charge_pump_currH|charge_pump_currL|PLL ON/OFF|control|ack|pwr_ready|clk_cource|00|powerH|powerL |freqH|freq |freq |freq |freqL|
//-----------------------------------------------------------------------------------------------------------------------

void usb_write(unsigned char number)												//priprava paketu pro odeslani pres USB do PC
{
	if(number == 0)																	//odeslani informace o aktualne nastavenem kmitoctu a vystupnim vykonu
	{
		tx_data[4] = frequency >> 32;
		tx_data[3] = frequency >> 24;
		tx_data[2] = frequency >> 16;
		tx_data[1] = frequency >> 8;
		tx_data[0] = frequency;
	
		tx_data[6] = out_power >> 8;
		tx_data[5] = out_power;		
	}
	else if((number >= 1) && (number <= 4))											//odeslani konkretni ulozene predvolby
	{
		tx_data[4] = frequency_presets[number - 1] >> 32;
		tx_data[3] = frequency_presets[number - 1] >> 24;
		tx_data[2] = frequency_presets[number - 1] >> 16;
		tx_data[1] = frequency_presets[number - 1] >> 8;
		tx_data[0] = frequency_presets[number - 1];
	
		tx_data[6] = frequency_presets[number + 3] >> 8;
		tx_data[5] = frequency_presets[number + 3];
	}
	else if((number >= 5) && (number <= 17))										//odeslani konkretni ulozene korekcni konstanty
	{
		tx_data[4] = 0;
		tx_data[3] = 0;
		tx_data[2] = 0;
		tx_data[1] = 0;
		tx_data[0] = 0;
	
		tx_data[6] = out_power_corr_const[number - 5] >> 8;
		tx_data[5] = out_power_corr_const[number - 5];
	}
	
	if(pll_mult_control == 0)														//detekce vypnute ci zapnute PLL
	{
		tx_data[7] = (clk_source)|(pwr_ready() << 1)|(ack << 2)|(control << 3)|		//odeslani ostatnich informaci (zdroj kmitoctu, ACK potvrzeni, ...)
			(DISABLE << 4)|(charge_pump_curr << 5)|(power_corr_control << 7);		
	}
	else
	{
		tx_data[7] = (clk_source)|(pwr_ready() << 1)|(ack << 2)|(control << 3)|		//odeslani ostatnich informaci (zdroj kmitoctu, ACK potvrzeni, ...)
			(ENABLE << 4)|(charge_pump_curr << 5)|(power_corr_control << 7);
	}
	
	if(usbInterruptIsReady())														//odeslani dat pres USB
	{																				//pouze pokud bylo odeslany predchozi data
   		unsigned char *p = tx_data;
		unsigned char len = sizeof(tx_data);										//maximalne 8 bajtu
    	if(len > 0) usbSetInterrupt(p, len);										//odeslat pouze pokud jsou k dispozici data
	}
}

ISR(TIMER2_OVF_vect)
{
	
}

void blink_led(void)																//obsluha LED "USB busy", tato funkce je volana ovladacem USB
{
	led_usb(LED_ON);
	_delay_ms(20);
	led_usb(LED_OFF);
}

int main(void)																		//main funkce
{	
	usbInit();																		//inicializace USB

    usbDeviceDisconnect();															//odpojeni USB										
    _delay_ms(250);																	//odpojeni USB na 250ms
    usbDeviceConnect();																//pripojeni USB
	
    sei();																			//povoleni preruseni
	
	uc_init();																		//inicializace uC
	
	led_a(LED_ON);																	//test LED
	led_b(LED_ON);
	led_usb(LED_ON);
	led_extclk(LED_ON);
	
	load_eeprom();																	//precteni obsahu EEPROM
//	frequency = frequency_presets[PRESET_1];										//nastaveni vychozi hodnoty kmitoctu
//	out_power = frequency_presets[PRESET_1 + 4];									//vykon nastaven na maximum
	
	frequency = 101000000000UL;														//frekvence 101 MHz
	out_power = MAX_OUT_POWER;														//vykon nastaven na maximum
	
	_delay_ms(1000);																//cekani 1s
	
	led_a(LED_OFF);																	//test LED
	led_b(LED_OFF);
	led_usb(LED_OFF);
	led_extclk(LED_OFF);
	
	dds_init();																		//inicializace DDS
	dds_set();																		//nastaveni DDS
	
	TIMSK |= (1 << TOIE2);															//povoleni preruseni po preteceni casovace 2
		
    while(TRUE)																		//hlavni nekonecna smycka
    {
		usbPoll();																	//nutne pro spravnou funkci USB
		
																					//funkce pro docasne uspani uC (nizsi ruseni)
		TCNT2 = 0;																	//reset casovace 2
		TCCR2 |= (1 << CS22)|(1 << CS21)|(1 << CS20);								//spusteni casovace 2, delicka 1024, preteceni za ~20ms
		
		set_sleep_mode(SLEEP_MODE_IDLE);											//mod IDLE
		sleep_mode();																//uspani uC
		
		TCCR2 &= ~((1 << CS22)|(1 << CS21)|(1 << CS20));							//zastaveni casovace
		
		if(pwr_ready() == 0) dds_init_request = 1;									//pokud je pristroj pripojen k USB a 12V, dojde po odpojeni a pripojeni 12V k nove inicializaci DDS
		else
		{
			if(dds_init_request)
			{
				dds_init();															//inicializace DDS
				dds_set();															//nastaveni DDS
				
				dds_init_request = 0;												//reset pozadavku na novou inicializaci
			}
		}
		
		if(usb_ready() == 0) control = HARDWARE;									//pokud je odpojeno USB, prejdi do harwaroveho rizeni
		
		if(control == HARDWARE)														//pokud je zvoleno hardwarove rizeni precti udaje z panelu
		{
			if(clk_source != button_extintclk())
			{
				clk_source = button_extintclk();
				
				dds_set();
			}
			
			preset_sel = rot_ctrl_sel();											//predvolba z ovladace A nebo B
			
			if(preset_sel == ROT_CTRL_A)											//rozsviceni LED A nebo B podle zvolene predvolby
			{
				led_a(LED_ON);
				led_b(LED_OFF);
			}
			else
			{
				led_a(LED_OFF);
				led_b(LED_ON);
			}

			preset_num = rot_ctrl_read(preset_sel) - 1;								//precteni polohy obladace

																					//nastaveni DDS pouze pokud doslo ke zmene			
			if((frequency != frequency_presets[preset_num]) || (out_power != frequency_presets[preset_num + 4]))
			{	
				frequency = frequency_presets[preset_num];
				out_power = frequency_presets[preset_num + 4];
							
				dds_set();															//nastaveni DDS
			}
		}
		else
		{
			led_a(LED_OFF);															//zhasnuti LED A a B v pripade softwaroveho ovladani
			led_b(LED_OFF);
		}
		
		if(new_usb_data_flag)														//pouze pokud jsou prijata nova data	
		{
			usb_read();																//zpracovani prijatych dat z PC
			new_usb_data_flag = 0;													//data zpracovana
		}
    }
}