/*
 * dds_lbr.c
 *
 * Created: 19.11.2011 20:50:15
 *  Author: Bus Ondrej
 */

#include "dds_lbr.h"

void dds_reset(void)													//reset AD9951
{
	DDS_PORT |= (1 << DDS_RESET_PIN);									//reset
	_delay_ms(5);
	DDS_PORT &= ~(1 << DDS_RESET_PIN);									//normalni stav
}

void dds_io_upd(void)													//aktualizace I/O registru
{
	DDS_PORT |= (1 << DDS_IO_UPD_PIN);									//vytvoreni nabezne hrany na pinu I/O update
	DELAY;
	DDS_PORT &= ~(1 << DDS_IO_UPD_PIN);
}

void dds_clock(void)													//vytvoreni hodinoveho pulzu na datove sbernici
{
	DELAY;
	DDS_PORT |= (1 << DDS_SCLK_PIN);
	DELAY;
	DDS_PORT &= ~(1 << DDS_SCLK_PIN);
}

void dds_init(void)														//inicializacni funkce AD9951
{
																		//nastaveni smeru portu
	DDS_DDR |= (1 << DDS_IO_UPD_PIN)|(1 << DDS_SDIO_PIN)|(1 << DDS_SCLK_PIN)|(1 << DDS_RESET_PIN);
	
																		//definovani vychozich stavu na sbernici
	DDS_PORT |= (1 << DDS_IO_UPD_PIN)|(1 << DDS_SDIO_PIN)|(1 << DDS_SCLK_PIN)|(1 << DDS_RESET_PIN);
	DELAY;
	DDS_PORT &= ~((1 << DDS_IO_UPD_PIN)|(1 << DDS_SDIO_PIN)|(1 << DDS_SCLK_PIN)|(1 << DDS_RESET_PIN));
	
	dds_reset();														//reset
	
	dds_data_send(INSTR_CFR1, SYNC_CLK_OFF | OSK_ENABLE, 4);			//deaktivace SYNC_CLK vystupu, umozneni zmeny amplitudy
	dds_multiplier(MULTIPLIER_PLL, CH_PUMP_150U);						//nastaveni nasobice
	dds_power_uw(MAX_OUT_POWER);										//nastaveni maximalni amplitudy
	dds_frequency(F_10_MHZ, 0);											//vystupni kmitocet 10 MHz
}
																		//odesle data do AD9951, vstupni parametry: instrukce, data, pocet bajtu
void dds_data_send(unsigned char instruction, unsigned long int data, unsigned char n_bytes)
{
	for(unsigned char i = 0; i < 8; i++)								//odeslani instrukce
	{
		if((instruction >> (7 - i)) & 0b00000001)
		{
			DDS_PORT |= (1 << DDS_SDIO_PIN);							//logicka 1			
		}
		else
		{
			DDS_PORT &= ~(1 << DDS_SDIO_PIN);							//logicka 0
		}
		
		dds_clock();													//hodinovy impulz
	}
	
	DDS_PORT &= ~(1 << DDS_SDIO_PIN);									//aktualizace dat
	
	for(unsigned char i = 0; i < (n_bytes * 8); i++)					//odeslani dat
	{
		if((data >> ((n_bytes * 8) - 1 - i)) & 0b00000001)
		{
			DDS_PORT |= (1 << DDS_SDIO_PIN);							//logicka 1		
		}
		else
		{
			DDS_PORT &= ~(1 << DDS_SDIO_PIN);							//logicka 0
		}
		
		dds_clock();													//hodinovy impulz
	}
	
	DDS_PORT &= ~(1 << DDS_SDIO_PIN);									//aktualizace dat
	
	dds_io_upd();														//update I/O
}

unsigned char dds_multiplier(unsigned char multiplier, unsigned char pump_curr)	//nastavi nasobic v rozsahu 4x az 20x, 0 = vypnut, druhy parametr je proud nabojovu pumpou (00=75uA, 01=100uA, 10=125uA, 11=150uA)
{
	if(multiplier == DISABLE)
	{
		dds_data_send(INSTR_CFR2, DISABLE, 3);							//odeslani, deaktivace PLL
		return 1;
	}
	
	if((multiplier < 4) || (multiplier > 20))	return 0;				//kontrola rozsahu
	
	multiplier <<= 3;
	multiplier |= VCO_RANGE_HIGH;										//nastaveni rozsahu VCO na 250 MHz az 400 MHz
	multiplier |= pump_curr;											//nastaveni proudu nabojove pumpy
	
	dds_data_send(INSTR_CFR2, multiplier, 3);							//odeslani
	
	return 1;
}

unsigned char dds_frequency(unsigned long long int freq, unsigned char fs)	//nastavi frekvenci v mHz
{
	freq = (freq + 50) / 100;											//uprava vstupni hodnoty podle minimalniho kroku DDS, zaokrouhlovani
	
																		//kontrola rozsahu frekvence	
	if((freq < (MIN_FREQ_HZ * 10)) || (freq > (MAX_FREQ_HZ * 10)))	return 0;
	
																		//vypocet ladiciho slova podle FTW=(f0*2^32)/fs
	freq <<= 32;														//freq=freq*2^32
	
	if(fs == 0)
	{
		freq = (freq / 10 + (FS_400_MHZ / 2)) / FS_400_MHZ;				//freq=freq/FS, vcetne zaokrouhlovani		
	}
	else
	{
		freq = (freq / 10 + (FS_320_MHZ / 2)) / FS_320_MHZ;				//freq=freq/FS, vcetne zaokrouhlovani		
	}

	
	dds_data_send(INSTR_FTW0, freq, 4);									//odeslani
	
	return 1;
}

unsigned char dds_power_uw(float out_power_uw)							//nastavi amplitudu v rozsahu 0 az 10000uW
{
	out_power_uw = sqrt(out_power_uw);
	out_power_uw *= MAX_AMPLITUDE;
	out_power_uw /= sqrt(MAX_OUT_POWER);
	
	if((unsigned int)(out_power_uw) > MAX_AMPLITUDE) return 0;			//kontrola rozsahu
	
	dds_data_send(INSTR_ASF, (unsigned int)(out_power_uw), 2);			//odeslani
	
	return 1;
}

void dds_powerdown(unsigned char enable_disable)						//powerdown mod, vstup ENABLE = powerdown, DISABLE = normalni
{
	if(enable_disable == ENABLE)
	{
		dds_data_send(INSTR_CFR1, POWERDOWN, 4);						//uspani AD9951
	}
	else
	{
		dds_init();														//inicializace = probuzeni
	}
}