/*
 * dds_lbr.h
 *
 * Created: 19.11.2011 20:50:28
 *  Author: Bus Ondrej
 */ 


#ifndef _DDS_LBR_H_
#define _DDS_LBR_H_

#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <util/delay.h>

#define DDS_PORT		PORTC								//PORT
#define DDS_DDR			DDRC								//DDR
#define DDS_RESET_PIN	3									//RESET pin
#define DDS_SCLK_PIN	2									//SCLK pin
#define DDS_SDIO_PIN	1									//SDIO pin
#define DDS_IO_UPD_PIN	0									//I/O update pin
#define	PULSE_TIME_US	100									//sirka datoveho impulsu v us
#define MULTIPLIER_PLL	5									//vychozi hodnota nasobice

#ifndef F_CPU
#define F_CPU 12000000UL									//taktovaci kmitocet procesoru (12 MHz)
#endif						

#define INSTR_FTW0		0x04								//instrukce: pristup do registru FTW0
#define INSTR_ASF		0x02								//instrukce: pristup do registru ASF
#define INSTR_CFR1		0x00								//instrukce: pristup do registru CFR1
#define INSTR_CFR2		0x01								//instrukce: pristup do registru CFR2
#define VCO_RANGE_HIGH	0x04								//nastavi rozsah VCO na 250 MHz az 400 MHz
#define CH_PUMP_75U		0x00								//proudy nabojove pumpy
#define CH_PUMP_100U	0x01
#define CH_PUMP_125U	0x02
#define CH_PUMP_150U	0x03
#define SYNC_CLK_OFF	0x02								//deaktivace SYNC_CLK vystupu
#define OSK_ENABLE		0x2000000							//OSK mod umozni zmenu amplitudy
#define POWERDOWN		0x9A4430							//powerdown
#define MAX_AMPLITUDE	0x3FFF								//maximani hodnota registru nastavujiciho amplitudu
#define MAX_OUT_POWER	10000U								//maximalni vystupni vykon v uW
#define F_10_MHZ		10000000000ULL						//10MHz
#define MIN_FREQ_HZ		1000000UL							//minimalni vystupni frekvence
#define MAX_FREQ_HZ		200000000UL							//maximalni vystupni frekvence
#define FS_400_MHZ		400000000UL							//hodinovy kmitocet AD9951
#define FS_320_MHZ		320000000UL
#define DELAY			_delay_us(PULSE_TIME_US)			//zpozdovaci funkce
#define ENABLE			1									//aktivovat
#define DISABLE			0									//deaktivovat

void dds_init(void);										//inicializace AD9951
void dds_reset(void);										//reset
unsigned char dds_multiplier(unsigned char multiplier,		//nastaveni nasobice (nasobici faktor a proud nabojove pumpy)
	unsigned char pump_curr);
unsigned char dds_frequency(unsigned long long int freq,	//nastaveni kmitoctu a vzorkovaci frekvence
	unsigned char fs);
unsigned char dds_power_uw(float out_power);				//nastaveni amplitudy v uW
void dds_powerdown(unsigned char enable_disable);			//powerdown mod
void dds_clock(void);										//hodinovy puls
void dds_io_upd(void);										//aktualizace registru
void dds_data_send(unsigned char instruction,				//odesle data do AD9951
unsigned long int data, unsigned char n_bytes);

#endif /* _DDS_LBR_H_ */