/*
 * uc_lbr.c
 *
 * Created: 20.11.2011 16:42:02
 *  Author: Bus Ondrej
 */

#include <avr/delay.h>
#include <avr/interrupt.h>
#include "uc_lbr.h"

ISR(TIMER1_OVF_vect)													//preruseni casovace 1
{
	CLK_CONTROL_DDR |= (1 << CLK_CONTROL_PIN);							//pin jako vystupni
	TCCR1B &= ~((1 << CS12)|(1 << CS11)|(1 << CS10));					//zastaveni casovace
}

unsigned char pwr_ready(void)											//vraci 1 v pripade, ze je pripojeno 12V
{
	return GET_BIT(PWR_READY_PORT, PWR_READY_PIN);
}

unsigned char usb_ready(void)											//vraci 1 v pripade, ze je pripojeno napajeni USB
{
	return GET_BIT(USB_READY_PORT, USB_READY_PIN);
}

void clk_control(unsigned char h_ext_l_int)								//prepina mezi internim a externim zdrojem kmitoctu (INTERNAL/EXTERNAL)
{
	if(h_ext_l_int == INTERNAL)
	{
		CLK_CONTROL_DDR &= ~(1 << CLK_CONTROL_PIN);						//pin jako vstupni
		CLK_CONTROL_PORT &= ~(1 << CLK_CONTROL_PIN);					//pull-up deaktivovan
		
		TIMSK |= (1 << TOIE1);											//aktivace preruseni po preteceni casovace 1
		TCNT1 = 0;														//reset TCNT1
		TCCR1B |= (1 << CS12)|(0 << CS11)|(0 << CS10);					//start casovace 1, preddelicka 256
	}
	else
	{
		CLK_CONTROL_DDR &= ~(1 << CLK_CONTROL_PIN);						//pin jako vstupni
		CLK_CONTROL_PORT |= (1 << CLK_CONTROL_PIN);						//pull-up aktivovan
		
		TIMSK |= (1 << TOIE1);											//aktivace preruseni po preteceni casovace 1
		TCNT1 = 0;														//reset TCNT1
		TCCR1B |= (1 << CS12)|(0 << CS11)|(0 << CS10);					//start casovace 1, preddelicka 256
	}
}

unsigned char rot_ctrl_read(unsigned char rot_ctrl)						//funkce vraci nastavene polohy na otocnych prepinacich, vstup ROT_CTRL_A, ROT_CTRL_B nebo ROT_CTRL_BOTH (ve formatu 0bBBBBAAAA), druhy vstup: posledni zapamatovana poloha prepinace
{
	unsigned char a = 1;
	unsigned char b = 1;
	
	ROT_CTRL_B_DDR &= ~(1 << ROT_CTRL_B_PIN);							//kontrolni pin B jako vstup bez pull-upu
	header_write(ROT_CTRL_B_H_PIN, PULL_UP_OFF);
	ROT_CTRL_A_DDR |= (1 << ROT_CTRL_A_PIN);							//kontrolni pin A jako vystup, uroven 0
	header_write(ROT_CTRL_A_H_PIN, 0);
	
	if(header_read(ROT_CTRL_A_1_H_PIN) == 0) a = 1;						//detekce polohy prepinace A
	if(header_read(ROT_CTRL_A_2_H_PIN) == 0) a = 2;
	if(header_read(ROT_CTRL_A_3_H_PIN) == 0) a = 3;
	if(header_read(ROT_CTRL_A_4_H_PIN) == 0) a = 4;
	
	ROT_CTRL_A_DDR &= ~(1 << ROT_CTRL_A_PIN);							//kontrolni pin A jako vstup bez pull-upu
	header_write(ROT_CTRL_A_H_PIN, PULL_UP_OFF);
	ROT_CTRL_B_DDR |= (1 << ROT_CTRL_B_PIN);							//kontrolni pin B jako vystup, uroven 0
	header_write(ROT_CTRL_B_H_PIN, 0);
	
	if(header_read(ROT_CTRL_B_1_H_PIN) == 0) b = 1;						//detekce polohy prepinace B
	if(header_read(ROT_CTRL_B_2_H_PIN) == 0) b = 2;
	if(header_read(ROT_CTRL_B_3_H_PIN) == 0) b = 3;
	if(header_read(ROT_CTRL_B_4_H_PIN) == 0) b = 4;
	
	ROT_CTRL_B_DDR &= ~(1 << ROT_CTRL_B_PIN);							//kontrolni pin B jako vstup bez pull-upu
	header_write(ROT_CTRL_B_H_PIN, PULL_UP_OFF);	

	if(rot_ctrl == ROT_CTRL_A) return a;								//vrati pouze stav prepinace A
	if(rot_ctrl == ROT_CTRL_B) return b;								//vrati pouze stav prepinace B
	
	return a | (b << 4);												//stav prepinacu ve formatu 0bBBBBAAAA
}

unsigned char rot_ctrl_sel(void)										//urcuje, zdali byla harwarove zvolena predvolba nastavena ovladacem A nebo B
{
	if(header_read(ROT_CTRL_SEL_H_PIN) == ROT_CTRL_A)
	{
		return ROT_CTRL_A;
	}	
	else
	{
		return ROT_CTRL_B;
	}	
}

void header_init(void)													//nastaveni smeru jednotlivych pinu na konektoru, prip. nastaveni pull-up rezistoru
{
	HEADER_5_DDR &= ~(1 << HEADER_5_PIN);								//nastaveni smeru nezapojenych pinu konektoru, pull-up aktivovan
	header_write(HEADER_5_PIN, PULL_UP_ON);
	HEADER_18_DDR &= ~(1 << HEADER_18_PIN);
	header_write(HEADER_18_PIN, PULL_UP_ON);
	
	ROT_CTRL_SEL_DDR &= ~(1 << ROT_CTRL_SEL_PIN);						//nastaveni smeru pinu pro vyber predvolby A nebo B

	ROT_CTRL_A_DDR &= ~(1 << ROT_CTRL_A_PIN);							//vychozi nastaveni smeru pinu pro cteni polohy otocnych prepinacu
	header_write(ROT_CTRL_A_H_PIN, PULL_UP_OFF);
	ROT_CTRL_B_DDR &= ~(1 << ROT_CTRL_B_PIN);
	header_write(ROT_CTRL_B_H_PIN, PULL_UP_OFF);
	ROT_CTRL_A_1_DDR &= ~(1 << ROT_CTRL_A_1_PIN);
	header_write(ROT_CTRL_A_1_H_PIN, PULL_UP_ON);
	ROT_CTRL_A_2_DDR &= ~(1 << ROT_CTRL_A_2_PIN);
	header_write(ROT_CTRL_A_2_H_PIN, PULL_UP_ON);
	ROT_CTRL_A_3_DDR &= ~(1 << ROT_CTRL_A_3_PIN);
	header_write(ROT_CTRL_A_3_H_PIN, PULL_UP_ON);
	ROT_CTRL_A_4_DDR &= ~(1 << ROT_CTRL_A_4_PIN);
	header_write(ROT_CTRL_A_4_H_PIN, PULL_UP_ON);
	
	BUT_EXTINTCLK_DDR &= ~(1 << BUT_EXTINTCLK_PIN);
	header_write(BUT_EXTINTCLK_H_PIN, PULL_UP_ON);
	
	LED_A_DDR |= (1 << LED_A_PIN);										//nastaveni smeru pinu pro ovladani LED A a B
	header_write(LED_A_H_PIN, LED_OFF);
	LED_B_DDR |= (1 << LED_B_PIN);
	header_write(LED_B_H_PIN, LED_OFF);
	LED_EXTCLOCK_DDR |= (1 << LED_EXTCLOCK_PIN);
	header_write(LED_EXTCLOCK_H_PIN, LED_OFF);
	LED_USB_DDR |= (1 << LED_USB_PIN);
	header_write(LED_USB_H_PIN, LED_OFF);	
}

unsigned char header_read(unsigned char pin)							//funkce pro cteni logicke urovne na jednotlivych pinech konektoru, pokud je zadan neexistujici ci nepouzitelny pin vraci 2
{
	switch(pin)
	{
		case 1:
			if(GET_BIT(HEADER_1_DDR, HEADER_1_PIN) == 0)				//jestlize je pin vstupni precte a vrati log uroven na pinu, jinak vrati chybovy stav 2
				return GET_BIT(HEADER_1_READ, HEADER_1_PIN);
			else return 2;
		case 2:
			if(GET_BIT(HEADER_2_DDR, HEADER_2_PIN) == 0)
				return GET_BIT(HEADER_2_READ, HEADER_2_PIN);
			else return 2;
		case 4:
			if(GET_BIT(HEADER_4_DDR, HEADER_4_PIN) == 0)
				return GET_BIT(HEADER_4_READ, HEADER_4_PIN);
			else return 2;
		case 5:
			if(GET_BIT(HEADER_5_DDR, HEADER_5_PIN) == 0)
				return GET_BIT(HEADER_5_READ, HEADER_5_PIN);
			else return 2;
		case 6:
			if(GET_BIT(HEADER_6_DDR, HEADER_6_PIN) == 0)
				return GET_BIT(HEADER_6_READ, HEADER_6_PIN);
			else return 2;
		case 7:
			if(GET_BIT(HEADER_7_DDR, HEADER_7_PIN) == 0)
				return GET_BIT(HEADER_7_READ, HEADER_7_PIN);
			else return 2;
		case 8:
			if(GET_BIT(HEADER_8_DDR, HEADER_8_PIN) == 0)
				return GET_BIT(HEADER_8_READ, HEADER_8_PIN);
			else return 2;
		case 13:
			if(GET_BIT(HEADER_13_DDR, HEADER_13_PIN) == 0)
				return GET_BIT(HEADER_13_READ, HEADER_13_PIN);
			else return 2;
		case 14:
			if(GET_BIT(HEADER_14_DDR, HEADER_14_PIN) == 0)
				return GET_BIT(HEADER_14_READ, HEADER_14_PIN);
			else return 2;
		case 15:
			if(GET_BIT(HEADER_15_DDR, HEADER_15_PIN) == 0)
				return GET_BIT(HEADER_15_READ, HEADER_15_PIN);
			else return 2;
		case 16:
			if(GET_BIT(HEADER_16_DDR, HEADER_16_PIN) == 0)
				return GET_BIT(HEADER_16_READ, HEADER_16_PIN);
			else return 2;
		case 17:
			if(GET_BIT(HEADER_17_DDR, HEADER_17_PIN) == 0)
				return GET_BIT(HEADER_17_READ, HEADER_17_PIN);
			else return 2;
		case 18:
			if(GET_BIT(HEADER_18_DDR, HEADER_18_PIN) == 0)
				return GET_BIT(HEADER_18_READ, HEADER_18_PIN);
			else return 2;
		case 19:
			if(GET_BIT(HEADER_19_DDR, HEADER_19_PIN) == 0)
				return GET_BIT(HEADER_19_READ, HEADER_19_PIN);
			else return 2;
		default:
			break;
	}
	
	return 2;
}

unsigned char header_write(unsigned char pin, unsigned char value)		//zapis logickych hodnot na jednotlive piny konektoru, lze pouzit take pro aktivaci pull-up rezistoru pokud je pin vstupni
{
	if(value > 1) return 0;
	
	switch(pin)
	{
		case 1:
			HEADER_1_WRITE = (HEADER_1_WRITE & ~(1 << HEADER_1_PIN)) | (value << HEADER_1_PIN);
			break;
		case 2:
			HEADER_2_WRITE = (HEADER_2_WRITE & ~(1 << HEADER_2_PIN)) | (value << HEADER_2_PIN);
			break;
		case 4:
			HEADER_4_WRITE = (HEADER_4_WRITE & ~(1 << HEADER_4_PIN)) | (value << HEADER_4_PIN);
			break;
		case 5:
			HEADER_5_WRITE = (HEADER_5_WRITE & ~(1 << HEADER_5_PIN)) | (value << HEADER_5_PIN);
			break;
		case 6:
			HEADER_6_WRITE = (HEADER_6_WRITE & ~(1 << HEADER_6_PIN)) | (value << HEADER_6_PIN);
			break;
		case 7:
			HEADER_7_WRITE = (HEADER_7_WRITE & ~(1 << HEADER_7_PIN)) | (value << HEADER_7_PIN);
			break;
		case 8:
			HEADER_8_WRITE = (HEADER_8_WRITE & ~(1 << HEADER_8_PIN)) | (value << HEADER_8_PIN);
			break;
		case 13:
			HEADER_13_WRITE = (HEADER_13_WRITE & ~(1 << HEADER_13_PIN)) | (value << HEADER_13_PIN);
			break;
		case 14:
			HEADER_14_WRITE = (HEADER_14_WRITE & ~(1 << HEADER_14_PIN)) | (value << HEADER_14_PIN);
			break;
		case 15:
			HEADER_15_WRITE = (HEADER_15_WRITE & ~(1 << HEADER_15_PIN)) | (value << HEADER_15_PIN);
			break;
		case 16:
			HEADER_16_WRITE = (HEADER_16_WRITE & ~(1 << HEADER_16_PIN)) | (value << HEADER_16_PIN);
			break;
		case 17:
			HEADER_17_WRITE = (HEADER_17_WRITE & ~(1 << HEADER_17_PIN)) | (value << HEADER_17_PIN);
			break;
		case 18:
			HEADER_18_WRITE = (HEADER_18_WRITE & ~(1 << HEADER_18_PIN)) | (value << HEADER_18_PIN);
			break;
		case 19:
			HEADER_19_WRITE = (HEADER_19_WRITE & ~(1 << HEADER_19_PIN)) | (value << HEADER_19_PIN);
			break;
		default:
			return 0;
	}
	
	return 1;
}

unsigned char button_extintclk(void)									//obsluha tlacitka pro volbu zdroje hodinoveho kmitoctu
{
	return header_read(BUT_EXTINTCLK_H_PIN);
}

void led_usb(unsigned char on_off)										//obsluha LED "USB busy"
{
	header_write(LED_USB_H_PIN, on_off);
}

void led_extclk(unsigned char on_off)									//obsluha LED "External clock"
{
	header_write(LED_EXTCLOCK_H_PIN, on_off);
}

void led_a(unsigned char on_off)										//obsluha LED "A"
{
	header_write(LED_A_H_PIN, on_off);
}

void led_b(unsigned char on_off)										//obsluha LED "B"
{
	header_write(LED_B_H_PIN, on_off);
}

void uc_init(void)														//inicializace uC ATMEGA16A
{
	DDRD &= ~((1 << 5)|(1 << 6));										//uzemnene piny jako vstupni
	DDRB &= ~(1 << 4);
	
	header_init();														//inicializace konektoru
	
	PWR_READY_DDR &= ~(1 << PWR_READY_PIN);								//nastaveni smeru pinu pro detekci 12V
	USB_READY_DDR &= ~(1 << USB_READY_PIN);								//nastaveni smeru pinu pro detekci USB napajeni
	clk_control(INTERNAL);												//vychozi nastaveni zdroje kmitoctu
}